"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const progress_1 = __importDefault(require("progress"));
const getPackageManager_1 = __importDefault(require("./getPackageManager"));
/**
 * Checks if the specified package will create a loop of peer dependencies by traversing all paths to find a cycle
 *
 * If a cycle was found, the offending peer dependency of the specified package is returned
 */
function isCircularPeer(peerDependencies, packageName) {
    let queue = [[packageName]];
    while (queue.length > 0) {
        const nextQueue = [];
        for (const path of queue) {
            const parents = Object.keys(peerDependencies[path[0]] ?? {});
            for (const name of parents) {
                if (name === path.at(-1)) {
                    return {
                        isCircular: true,
                        offendingPackage: path[0],
                    };
                }
                nextQueue.push([name, ...path]);
            }
        }
        queue = nextQueue;
    }
    return {
        isCircular: false,
    };
}
/**
 * Get the latest or greatest versions from the NPM repository based on the version target.
 *
 * @param packageMap   An object whose keys are package name and values are version
 * @param [options={}] Options.
 * @returns Promised {packageName: peer dependencies} collection
 */
async function getPeerDependenciesFromRegistry(packageMap, options) {
    const packageManager = (0, getPackageManager_1.default)(options, options.packageManager);
    if (!packageManager.getPeerDependencies)
        return {};
    const numItems = Object.keys(packageMap).length;
    let bar;
    if (!options.json && options.loglevel !== 'silent' && options.loglevel !== 'verbose' && numItems > 0) {
        bar = new progress_1.default('[:bar] :current/:total :percent', { total: numItems, width: 20 });
        bar.render();
    }
    const peerDependencies = Object.entries(packageMap).reduce(async (accumPromise, [pkg, version]) => {
        const dep = await packageManager.getPeerDependencies(pkg, version);
        if (bar) {
            bar.tick();
        }
        const accum = await accumPromise;
        const newAcc = { ...accum, [pkg]: dep };
        const circularData = isCircularPeer(newAcc, pkg);
        if (circularData.isCircular) {
            delete newAcc[pkg][circularData.offendingPackage];
        }
        return newAcc;
    }, {});
    return peerDependencies;
}
exports.default = getPeerDependenciesFromRegistry;
//# sourceMappingURL=getPeerDependenciesFromRegistry.js.map